import { ɵɵdefineInjectable, ɵɵinject, Injectable, EventEmitter, SimpleChange, Component, ViewEncapsulation, ElementRef, Host, Input, Output, ViewChild, HostBinding, SecurityContext, NgModule } from '@angular/core';
import { MatIconRegistry, MatIconModule } from '@angular/material/icon';
import { MAT_TOOLTIP_DEFAULT_OPTIONS, MatTooltipModule } from '@angular/material/tooltip';
import { EventManager, DomSanitizer, BrowserModule } from '@angular/platform-browser';
import { MarkdownEditor, DEFAULT_OPTIONS } from '@mdefy/markdown-editor-core';
import { MarkdownService, SECURITY_CONTEXT, MarkdownModule } from 'ngx-markdown';
import { Observable, Subject } from 'rxjs';
import { startWith, map, takeUntil } from 'rxjs/operators';
import { CommonModule } from '@angular/common';
import { HttpClientModule } from '@angular/common/http';
import { MatButtonModule } from '@angular/material/button';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatSelectModule } from '@angular/material/select';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';

/**
 * An injectable hotkeys service to add keybindings.
 */
class Keybindings {
    constructor(eventManager) {
        this.eventManager = eventManager;
    }
    /**
     * Adds an _keydown_ event listener to the Angular `EventManager` with the specified
     * `keys`. The listener is applied to specified `element`. Returns an RxJS observable
     * of the keyboard event.
     *
     * @param element The HTML element to which the keybinding shall be applied to.
     * @param keys The key combination which shall trigger the event.
     */
    addKeybinding(element, keys) {
        const event = `keydown.${keys}`;
        return new Observable((observer) => {
            const handler = (e) => {
                e.preventDefault();
                observer.next(e);
            };
            const dispose = this.eventManager.addEventListener(element, event, handler);
            return () => {
                dispose();
            };
        });
    }
}
Keybindings.ɵprov = ɵɵdefineInjectable({ factory: function Keybindings_Factory() { return new Keybindings(ɵɵinject(EventManager)); }, token: Keybindings, providedIn: "root" });
Keybindings.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
Keybindings.ctorParameters = () => [
    { type: EventManager }
];

/**
 * Transforms a _CodeMirror_ event to an RxJS observable.
 *
 * @param cm the `CodeMirror.Editor` instance of which the event shall be observed
 * @param eventName the name of a _CodeMirror_ event
 */
function fromCmEvent(cm, eventName) {
    return new Observable((subscriber) => {
        let handler;
        switch (eventName) {
            case 'change':
                handler = (...args) => subscriber.next({ instance: args[0], changeObj: args[1] });
                break;
            case 'changes':
                handler = (...args) => subscriber.next({ instance: args[0], changes: args[1] });
                break;
            case 'beforeChange':
                handler = (...args) => subscriber.next({ instance: args[0], changeObj: args[1] });
                break;
            case 'cursorActivity':
                handler = (...args) => subscriber.next({ instance: args[0] });
                break;
            case 'keyHandled':
                handler = (...args) => subscriber.next({ instance: args[0], name: args[1], event: args[2] });
                break;
            case 'inputRead':
                handler = (...args) => subscriber.next({ instance: args[0], changeObj: args[1] });
                break;
            case 'electricInput':
                handler = (...args) => subscriber.next({ instance: args[0], line: args[1] });
                break;
            case 'beforeSelectionChange':
                handler = (...args) => subscriber.next({ instance: args[0], obj: args[1] });
                break;
            case 'viewportChange':
                handler = (...args) => subscriber.next({ instance: args[0], from: args[1], to: args[2] });
                break;
            case 'swapDoc':
                handler = (...args) => subscriber.next({ instance: args[0], oldDoc: args[1] });
                break;
            case 'gutterClick':
                handler = (...args) => subscriber.next({ instance: args[0], line: args[1], gutter: args[2], clickEvent: args[3] });
                break;
            case 'gutterContextMenu':
                handler = (...args) => subscriber.next({ instance: args[0], line: args[1], gutter: args[2], contextMenu: args[3] });
                break;
            case 'focus':
                handler = (...args) => subscriber.next({ instance: args[0], event: args[1] });
                break;
            case 'blur':
                handler = (...args) => subscriber.next({ instance: args[0], event: args[1] });
                break;
            case 'scroll':
                handler = (...args) => subscriber.next({ instance: args[0] });
                break;
            case 'refresh':
                handler = (...args) => subscriber.next({ instance: args[0] });
                break;
            case 'optionChange':
                handler = (...args) => subscriber.next({ instance: args[0], option: args[1] });
                break;
            case 'scrollCursorIntoView':
                handler = (...args) => subscriber.next({ instance: args[0], event: args[1] });
                break;
            case 'update':
                handler = (...args) => subscriber.next({ instance: args[0] });
                break;
            case 'renderLine':
                handler = (...args) => subscriber.next({ instance: args[0], line: args[1], element: args[2] });
                break;
            case 'overwriteToggle':
                handler = (...args) => subscriber.next({ instance: args[0], overwrite: args[1] });
                break;
            default:
                handler = (...args) => subscriber.next({ instance: args[0], event: args[1] });
        }
        if (cm) {
            cm.on(eventName, handler);
            return () => cm.off(eventName, handler);
        }
        subscriber.error(new Error('CodeMirror instance is undefined'));
        return;
    });
}

class StatusbarService {
    constructor() {
        /**
         * The default statusbar setup.
         */
        this.DEFAULT_STATUSBAR = ['wordCount', 'characterCount', '|', 'cursorPosition'];
    }
    /**
     * The default configurations of all items.
     */
    get DEFAULT_ITEMS() {
        return this._defaultItems;
    }
    /**
     * Returns the default configuration of the item with the specified name.
     * Returns `undefined`, if no item with the specified name can be found.
     */
    getDefaultItem(itemName) {
        return this.DEFAULT_ITEMS.find((i) => i.name === itemName);
    }
    /**
     * Defines the default statusbar items.
     * Cannot be done statically as the values depend on the `MarkdownEditor` instance.
     */
    defineDefaultItems(mde) {
        const defaultItems = [
            {
                name: 'wordCount',
                value: fromCmEvent(mde.cm, 'changes').pipe(startWith(true), map(() => 'Words: ' + mde.getWordCount().toString())),
            },
            {
                name: 'characterCount',
                value: fromCmEvent(mde.cm, 'changes').pipe(startWith(true), map(() => 'Characters: ' + mde.getCharacterCount().toString())),
            },
            {
                name: 'cursorPosition',
                value: fromCmEvent(mde.cm, 'cursorActivity').pipe(startWith(true), map(() => {
                    const pos = mde.getCursorPos();
                    return `${pos.line}:${pos.ch}`;
                })),
            },
            // Normalize separator item to reduce type complexity in template.
            // Effectively, only the `name` property is needed.
            {
                name: '|',
                value: new Observable(),
            },
        ];
        this._defaultItems = defaultItems;
    }
}
StatusbarService.decorators = [
    { type: Injectable }
];

class ToolbarService {
    constructor() {
        /**
         * The default toolbar setup.
         */
        this.DEFAULT_TOOLBAR = [
            'setHeadingLevel',
            'toggleHeadingLevel',
            'increaseHeadingLevel',
            'decreaseHeadingLevel',
            'toggleBold',
            'toggleItalic',
            'toggleStrikethrough',
            '|',
            'toggleUnorderedList',
            'toggleOrderedList',
            'toggleCheckList',
            '|',
            'toggleQuote',
            'toggleInlineCode',
            'insertCodeBlock',
            '|',
            'insertLink',
            'insertImageLink',
            'insertTable',
            'insertHorizontalRule',
            '|',
            'toggleRichTextMode',
            'formatContent',
            '|',
            'downloadAsFile',
            'importFromFile',
            '|',
            'togglePreview',
            'toggleSideBySidePreview',
            '|',
            'undo',
            'redo',
            '|',
            'openMarkdownGuide',
        ];
    }
    /**
     * The default configurations of all items
     */
    get DEFAULT_ITEMS() {
        return this._defaultItems;
    }
    /**
     * Returns the default configuration of the item with the specified name.
     * Returns `undefined`, if no item with the specified name can be found.
     */
    getDefaultItem(itemName) {
        return this.DEFAULT_ITEMS.find((i) => i.name === itemName);
    }
    /**
     * Defines the default toolbar items.
     * Cannot be done statically as the actions depend on the `MarkdownEditorComponent` instance.
     */
    defineDefaultItems(ngxMde) {
        const defaultItems = [
            {
                name: 'setHeadingLevel',
                action: (level) => ngxMde.mde.setHeadingLevel(level),
                shortcut: 'Shift-Ctrl-Alt-H',
                isActive: () => {
                    if (!ngxMde.mde.hasTokenAtCursorPos('header'))
                        return 0;
                    const token = ngxMde.mde.cm.getTokenAt(ngxMde.mde.getCursorPos());
                    return token.state.base.header;
                },
                tooltip: 'Set Heading Level',
                icon: {
                    format: 'svgString',
                    iconName: 'format_heading',
                    svgHtmlString: FORMAT_HEADING,
                },
                disableOnPreview: true,
            },
            {
                name: 'toggleHeadingLevel',
                action: () => ngxMde.mde.increaseHeadingLevel(),
                tooltip: 'Heading',
                shortcut: 'Alt-H',
                icon: {
                    format: 'svgString',
                    iconName: 'format_heading',
                    svgHtmlString: FORMAT_HEADING,
                },
                disableOnPreview: true,
            },
            {
                name: 'increaseHeadingLevel',
                action: () => ngxMde.mde.increaseHeadingLevel(),
                tooltip: 'Smaller Heading',
                icon: {
                    format: 'svgString',
                    iconName: 'format_heading_decrease',
                    svgHtmlString: FORMAT_HEADING_SMALLER,
                },
                disableOnPreview: true,
            },
            {
                name: 'decreaseHeadingLevel',
                action: () => ngxMde.mde.decreaseHeadingLevel(),
                tooltip: 'Bigger Heading',
                icon: {
                    format: 'svgString',
                    iconName: 'format_heading_increase',
                    svgHtmlString: FORMAT_HEADING_BIGGER,
                },
                disableOnPreview: true,
            },
            {
                name: 'toggleBold',
                action: () => ngxMde.mde.toggleBold(),
                isActive: () => ngxMde.mde.hasTokenAtCursorPos('strong'),
                tooltip: 'Toggle Bold',
                icon: {
                    format: 'material',
                    iconName: 'format_bold',
                },
                disableOnPreview: true,
            },
            {
                name: 'toggleItalic',
                action: () => ngxMde.mde.toggleItalic(),
                isActive: () => ngxMde.mde.hasTokenAtCursorPos('em'),
                tooltip: 'Toggle Italic',
                icon: {
                    format: 'material',
                    iconName: 'format_italic',
                },
                disableOnPreview: true,
            },
            {
                name: 'toggleStrikethrough',
                action: () => ngxMde.mde.toggleStrikethrough(),
                isActive: () => ngxMde.mde.hasTokenAtCursorPos('strikethrough'),
                tooltip: 'Toggle Strikethrough',
                icon: {
                    format: 'material',
                    iconName: 'format_strikethrough',
                },
                disableOnPreview: true,
            },
            {
                name: 'toggleUnorderedList',
                action: () => ngxMde.mde.toggleUnorderedList(),
                isActive: () => this.isListTypeActive(ngxMde, 'unordered'),
                tooltip: 'Toggle Unordered List',
                icon: {
                    format: 'material',
                    iconName: 'format_list_bulleted',
                },
                disableOnPreview: true,
            },
            {
                name: 'toggleOrderedList',
                action: () => ngxMde.mde.toggleOrderedList(),
                isActive: () => this.isListTypeActive(ngxMde, 'ordered'),
                tooltip: 'Toggle Ordered List',
                icon: {
                    format: 'material',
                    iconName: 'format_list_numbered',
                },
                disableOnPreview: true,
            },
            {
                name: 'toggleCheckList',
                action: () => ngxMde.mde.toggleCheckList(),
                isActive: () => this.isListTypeActive(ngxMde, 'check'),
                tooltip: 'Toggle Checklist',
                icon: {
                    format: 'material',
                    iconName: 'check_box',
                },
                disableOnPreview: true,
            },
            {
                name: 'toggleQuote',
                action: () => ngxMde.mde.toggleQuote(),
                isActive: () => ngxMde.mde.hasTokenAtCursorPos('quote'),
                tooltip: 'Toggle Quotation',
                icon: {
                    format: 'material',
                    iconName: 'format_quote',
                },
                disableOnPreview: true,
            },
            {
                name: 'toggleInlineCode',
                action: () => ngxMde.mde.toggleInlineCode(),
                isActive: () => this.isCodeTypeActive(ngxMde, 'inline'),
                tooltip: 'Toggle Inline Code',
                icon: {
                    format: 'material',
                    iconName: 'code',
                },
                disableOnPreview: true,
            },
            {
                name: 'insertCodeBlock',
                action: () => ngxMde.mde.insertCodeBlock(),
                isActive: () => this.isCodeTypeActive(ngxMde, 'block'),
                tooltip: 'Insert Code Block',
                icon: {
                    format: 'svgString',
                    iconName: 'file_code',
                    svgHtmlString: FILE_CODE,
                },
                disableOnPreview: true,
            },
            {
                name: 'insertLink',
                action: () => ngxMde.mde.insertLink(),
                isActive: () => (ngxMde.mde.hasTokenAtCursorPos('link-text') || ngxMde.mde.hasTokenAtCursorPos('link')) &&
                    !ngxMde.mde.hasTokenAtCursorPos('image'),
                tooltip: 'Insert Link',
                icon: {
                    format: 'material',
                    iconName: 'insert_link',
                },
                disableOnPreview: true,
            },
            {
                name: 'insertImageLink',
                action: () => ngxMde.mde.insertImageLink(),
                isActive: () => ngxMde.mde.hasTokenAtCursorPos('image'),
                tooltip: 'Insert Image Link',
                icon: {
                    format: 'material',
                    iconName: 'image',
                },
                disableOnPreview: true,
            },
            {
                name: 'insertTable',
                action: () => ngxMde.mde.insertTable(),
                tooltip: 'Insert Table',
                icon: {
                    format: 'material',
                    iconName: 'table_chart',
                },
                disableOnPreview: true,
            },
            {
                name: 'insertHorizontalRule',
                action: () => ngxMde.mde.insertHorizontalRule(),
                isActive: () => ngxMde.mde.hasTokenAtCursorPos('hr'),
                tooltip: 'Insert Horizontal Rule',
                icon: {
                    format: 'material',
                    iconName: 'horizontal_rule',
                },
                disableOnPreview: true,
            },
            {
                name: 'toggleRichTextMode',
                action: () => ngxMde.mde.toggleRichTextMode(),
                isActive: () => {
                    const mode = ngxMde.mde.cm.getOption('mode');
                    return mode === 'gfm' || mode.name === 'gfm';
                },
                tooltip: 'Toggle Rich-Text Mode',
                icon: {
                    format: 'material',
                    iconName: 'highlight',
                },
                disableOnPreview: true,
            },
            {
                name: 'formatContent',
                action: () => ngxMde.mde.formatContent(),
                tooltip: 'Format Content',
                icon: {
                    format: 'material',
                    iconName: 'format_paint',
                },
                disableOnPreview: true,
            },
            {
                name: 'downloadAsFile',
                action: () => ngxMde.mde.downloadAsFile(),
                tooltip: 'Download As File',
                icon: {
                    format: 'material',
                    iconName: 'get_app',
                },
                disableOnPreview: true,
            },
            {
                name: 'importFromFile',
                action: () => ngxMde.mde.importFromFile(),
                tooltip: 'Import From File',
                icon: {
                    format: 'svgString',
                    iconName: 'upload',
                    svgHtmlString: UPLOAD,
                },
                disableOnPreview: true,
            },
            {
                name: 'togglePreview',
                action: () => ngxMde.togglePreview(),
                shortcut: 'Alt-P',
                isActive: () => ngxMde.showPreview,
                tooltip: 'Toggle Preview',
                icon: {
                    format: 'material',
                    iconName: 'preview',
                },
                disableOnPreview: false,
            },
            {
                name: 'toggleSideBySidePreview',
                action: () => ngxMde.toggleSideBySidePreview(),
                shortcut: 'Shift-Alt-P',
                isActive: () => ngxMde.showSideBySidePreview,
                tooltip: 'Toggle Side-by-Side Preview',
                icon: {
                    format: 'svgString',
                    iconName: 'column',
                    svgHtmlString: COLUMN,
                },
                disableOnPreview: false,
            },
            {
                name: 'undo',
                action: () => ngxMde.mde.undo(),
                tooltip: 'Undo',
                icon: {
                    format: 'material',
                    iconName: 'undo',
                },
                disableOnPreview: true,
            },
            {
                name: 'redo',
                action: () => ngxMde.mde.redo(),
                shortcut: 'Ctrl-S',
                tooltip: 'Redo',
                icon: {
                    format: 'material',
                    iconName: 'redo',
                },
                disableOnPreview: true,
            },
            {
                name: 'openMarkdownGuide',
                action: () => ngxMde.mde.openMarkdownGuide(),
                tooltip: 'Open Markdown Guide',
                icon: {
                    format: 'material',
                    iconName: 'help',
                },
                disableOnPreview: false,
            },
            // Normalize separator item to reduce type complexity in template.
            // Effectively, only the `name` property is needed.
            {
                name: '|',
                action: () => { },
                tooltip: '',
                icon: { format: 'material', iconName: '' },
                disableOnPreview: false,
            },
        ];
        this._defaultItems = defaultItems;
    }
    isListTypeActive(ngxMde, listType) {
        const isList = ngxMde.mde.hasTokenAtCursorPos('list');
        if (!isList)
            return false;
        const selections = ngxMde.mde.cm.listSelections();
        let isListType = false;
        if (selections === null || selections === void 0 ? void 0 : selections.length) {
            const lineNumber = selections[selections.length - 1].from().line;
            isListType = ngxMde.mde.getListTypeOfLine(lineNumber) === listType;
        }
        return isListType;
    }
    isCodeTypeActive(ngxMde, codeType) {
        const isCode = ngxMde.mde.hasTokenAtCursorPos('code');
        if (!isCode)
            return false;
        const token = ngxMde.mde.cm.getTokenAt(ngxMde.mde.getCursorPos());
        if (codeType === 'block') {
            return token.state.overlay.codeBlock;
        }
        else {
            return token.state.overlay.code;
        }
    }
}
ToolbarService.decorators = [
    { type: Injectable }
];
/* eslint-disable max-len */
const COLUMN = `
  <!-- Icon from Font Awesome: https://fontawesome.com/icons/columns?style=solid; License: https://fontawesome.com/license -->
  <svg
    aria-hidden="true"
    focusable="false"
    data-prefix="fas"
    data-icon="columns"
    class="svg-inline--fa fa-columns fa-w-16"
    role="img"
    xmlns="http://www.w3.org/2000/svg"
    viewBox="0 0 512 512"
    licenseUrl="https://fontawesome.com/license"
  >
    <path
      fill="currentColor"
      d="M464 32H48C21.49 32 0 53.49 0 80v352c0 26.51 21.49 48 48 48h416c26.51 0 48-21.49 48-48V80c0-26.51-21.49-48-48-48zM224 416H64V160h160v256zm224 0H288V160h160v256z"
    ></path>
  </svg>
`;
const FILE_CODE = `
  <!-- Icon from Font Awesome: https://fontawesome.com/icons/file-code?style=solid; License: https://fontawesome.com/license -->
  <svg
    aria-hidden="true"
    focusable="false"
    data-prefix="fas"
    data-icon="file-code"
    class="svg-inline--fa fa-file-code fa-w-12"
    role="img"
    xmlns="http://www.w3.org/2000/svg"
    viewBox="0 0 384 512"
    licenseUrl="https://fontawesome.com/license"
  >
    <path
      fill="currentColor"
      d="M384 121.941V128H256V0h6.059c6.365 0 12.47 2.529 16.971 7.029l97.941 97.941A24.005 24.005 0 0 1 384 121.941zM248 160c-13.2 0-24-10.8-24-24V0H24C10.745 0 0 10.745 0 24v464c0 13.255 10.745 24 24 24h336c13.255 0 24-10.745 24-24V160H248zM123.206 400.505a5.4 5.4 0 0 1-7.633.246l-64.866-60.812a5.4 5.4 0 0 1 0-7.879l64.866-60.812a5.4 5.4 0 0 1 7.633.246l19.579 20.885a5.4 5.4 0 0 1-.372 7.747L101.65 336l40.763 35.874a5.4 5.4 0 0 1 .372 7.747l-19.579 20.884zm51.295 50.479l-27.453-7.97a5.402 5.402 0 0 1-3.681-6.692l61.44-211.626a5.402 5.402 0 0 1 6.692-3.681l27.452 7.97a5.4 5.4 0 0 1 3.68 6.692l-61.44 211.626a5.397 5.397 0 0 1-6.69 3.681zm160.792-111.045l-64.866 60.812a5.4 5.4 0 0 1-7.633-.246l-19.58-20.885a5.4 5.4 0 0 1 .372-7.747L284.35 336l-40.763-35.874a5.4 5.4 0 0 1-.372-7.747l19.58-20.885a5.4 5.4 0 0 1 7.633-.246l64.866 60.812a5.4 5.4 0 0 1-.001 7.879z"
    ></path>
  </svg>
`;
const FORMAT_HEADING = `
  <!-- Icon from Font Awesome: https://fontawesome.com/icons/heading?style=solid; License: https://fontawesome.com/license -->
  <svg
    aria-hidden="true"
    focusable="false"
    data-prefix="fas"
    data-icon="heading"
    class="svg-inline--fa fa-heading fa-w-16"
    role="img"
    xmlns="http://www.w3.org/2000/svg"
    width="100%"
    height="100%"
    viewBox="0 0 512 512"
    licenseUrl="https://fontawesome.com/license"
  >
    <path
      fill="currentColor"
      d="M448 96v320h32a16 16 0 0 1 16 16v32a16 16 0 0 1-16 16H320a16 16 0 0 1-16-16v-32a16 16 0 0 1 16-16h32V288H160v128h32a16 16 0 0 1 16 16v32a16 16 0 0 1-16 16H32a16 16 0 0 1-16-16v-32a16 16 0 0 1 16-16h32V96H32a16 16 0 0 1-16-16V48a16 16 0 0 1 16-16h160a16 16 0 0 1 16 16v32a16 16 0 0 1-16 16h-32v128h192V96h-32a16 16 0 0 1-16-16V48a16 16 0 0 1 16-16h160a16 16 0 0 1 16 16v32a16 16 0 0 1-16 16z"
    ></path>
  </svg>
`;
const FORMAT_HEADING_BIGGER = `
  <!-- Icon from Font Awesome: https://fontawesome.com/icons/heading?style=solid; License: https://fontawesome.com/license -->
  <svg
    aria-hidden="true"
    focusable="false"
    data-prefix="fas"
    data-icon="heading"
    class="svg-inline--fa fa-heading fa-w-16"
    role="img"
    xmlns="http://www.w3.org/2000/svg"
    width="100%"
    height="100%"
    viewBox="0 0 720 720"
    licenseUrl="https://fontawesome.com/license"
  >
    <path
      fill="currentColor"
      d="M448 200v320h32a16 16 0 0 1 16 16v32a16 16 0 0 1-16 16H320a16 16 0 0 1-16-16v-32a16 16 0 0 1 16-16h32V392H160v128h32a16 16 0 0 1 16 16v32a16 16 0 0 1-16 16H32a16 16 0 0 1-16-16v-32a16 16 0 0 1 16-16h32V200H32a16 16 0 0 1-16-16V152a16 16 0 0 1 16-16h160a16 16 0 0 1 16 16v32a16 16 0 0 1-16 16h-32v128h192V200h-32a16 16 0 0 1-16-16V152a16 16 0 0 1 16-16h160a16 16 0 0 1 16 16v32a16 16 0 0 1-16 16z"
    ></path>
    <path
      fill="currentColor"
      d="M620 285 l87 150 h-174 z"
    ></path>
  </svg>
`;
const FORMAT_HEADING_SMALLER = `
  <!-- Icon from Font Awesome: https://fontawesome.com/icons/heading?style=solid; License: https://fontawesome.com/license -->
  <svg
    aria-hidden="true"
    focusable="false"
    data-prefix="fas"
    data-icon="heading"
    class="svg-inline--fa fa-heading fa-w-16"
    role="img"
    xmlns="http://www.w3.org/2000/svg"
    width="100%"
    height="100%"
    viewBox="0 0 720 720"
    licenseUrl="https://fontawesome.com/license"
  >
    <path
      fill="currentColor"
      d="M448 200v320h32a16 16 0 0 1 16 16v32a16 16 0 0 1-16 16H320a16 16 0 0 1-16-16v-32a16 16 0 0 1 16-16h32V392H160v128h32a16 16 0 0 1 16 16v32a16 16 0 0 1-16 16H32a16 16 0 0 1-16-16v-32a16 16 0 0 1 16-16h32V200H32a16 16 0 0 1-16-16V152a16 16 0 0 1 16-16h160a16 16 0 0 1 16 16v32a16 16 0 0 1-16 16h-32v128h192V200h-32a16 16 0 0 1-16-16V152a16 16 0 0 1 16-16h160a16 16 0 0 1 16 16v32a16 16 0 0 1-16 16z"
    ></path>
    <path
      fill="currentColor"
      d="M620 435 l87 -150 h-174 z"
    ></path>
  </svg>
`;
const UPLOAD = `
  <svg
    focusable="false"
    data-icon="upload"
    role="img"
    xmlns="http://www.w3.org/2000/svg"
    width="100%"
    height="100%"
    viewBox="2 2 20 20"
  >
    <path d="M9 16h6v-6h4l-7-7-7 7h4zm-4 2h14v2H5z" />
  </svg>
`;

/**
 * An event emitter that is able to transform an RxJS observable
 * into an Angular event.
 */
class ObservableEmitter extends EventEmitter {
    /**
     * Subscribes to the specified RxJS observable and emits an event
     * containing the observed value. Unsubscribes the passed observable
     * when the event is unsubscribed.
     */
    emitObservable(o) {
        const subscription = o.subscribe((x) => this.emit(x));
        this.subscribe(() => { }, undefined, () => subscription.unsubscribe());
    }
}

const markdownEditorTooltipDefaults = {
    showDelay: 1000,
    hideDelay: 0,
    touchendHideDelay: 1000,
};
const ɵ0 = markdownEditorTooltipDefaults;
class MarkdownEditorComponent {
    constructor(iconRegistry, domSanitizer, hotkeys, hostElement, markdownService, toolbarService, statusbarService) {
        this.iconRegistry = iconRegistry;
        this.domSanitizer = domSanitizer;
        this.hotkeys = hotkeys;
        this.hostElement = hostElement;
        this.markdownService = markdownService;
        this.toolbarService = toolbarService;
        this.statusbarService = statusbarService;
        /**
         * Data string to set as content of the editor.
         */
        this.data = '';
        /**
         * Options to configure _Ngx Markdown Editor_.
         *
         * Basically `MarkdownEditorOptions` from _Markdown Editor Core_ are forwarded,
         * including some adjustments and extensions.
         */
        this.options = {};
        /**
         * Custom set of toolbar items.
         */
        this.toolbar = [];
        /**
         * Custom set of statusbar items.
         */
        this.statusbar = [];
        /**
         * The current language applied to internationalized items.
         */
        this.language = 'en';
        /**
         * Specifies whether the editor is a required form field. An asterisk will be appended to the label.
         */
        this.required = false;
        /**
         * Specifies whether and which Angular Material style is used.
         */
        this.materialStyle = false;
        /**
         * Specifies whether the editor is disabled.
         */
        this.disabled = false;
        /**
         * Specifies whether the toolbar is rendered.
         */
        this.showToolbar = true;
        /**
         * Specifies whether the statusbar is rendered.
         */
        this.showStatusbar = true;
        /**
         * Specifies whether tooltips are shown for toolbar items.
         */
        this.showTooltips = true;
        /**
         * Specifies whether the key combination is included in the tooltip.
         */
        this.shortcutsInTooltips = true;
        /**
         * Emits when the editor's content changes.
         */
        this.contentChange = new ObservableEmitter();
        /**
         * Emits when the editor's cursor is moved.
         */
        this.cursorActivity = new ObservableEmitter();
        /**
         * Emits when the editor receives focus.
         */
        this.editorFocus = new ObservableEmitter();
        /**
         * Emits when the editor loses focus.
         */
        this.editorBlur = new ObservableEmitter();
        /**
         * _Not intended to be used outside the component. Only made public for access inside template._
         */
        this.normalizedToolbarItems = [];
        /**
         * _Not intended to be used outside the component. Only made public for access inside template._
         */
        this.activeToolbarItems = [];
        /**
         * _Not intended to be used outside the component. Only made public for access inside template._
         */
        this.toolbarItemTooltips = [];
        /**
         * _Not intended to be used outside the component. Only made public for access inside template._
         */
        this.normalizedStatusbarItems = [];
        /**
         * _Not intended to be used outside the component. Only made public for access inside template._
         */
        this.showPreview = false;
        /**
         * _Not intended to be used outside the component. Only made public for access inside template._
         */
        this.showSideBySidePreview = false;
        /**
         * _Not intended to be used outside the component. Only made public for access inside template._
         */
        this.blockBlur = false;
        /**
         * _Not intended to be used outside the component. Only made public for access inside template._
         */
        this.focused = false;
        this.shortcutResetter = new Subject();
        // Render checkbox dummies which can be replaced by an `<input type="checkbox"> later,
        // because the checkboxes rendered by marked.js inside ngx-markdown are removed by Angular sanitizer.
        this.markdownService.renderer.checkbox = (checked) => (checked ? '[x] ' : '[ ] ');
    }
    get disabledStyle() {
        return this.disabled;
    }
    get default() {
        return !this.options.editorThemes && !this.materialStyle;
    }
    get material() {
        return this.materialStyle;
    }
    get class() {
        return this.options.editorThemes;
    }
    get appearanceStandard() {
        return this.materialStyle === true || this.materialStyle === 'standard';
    }
    get appearanceFill() {
        return this.materialStyle === 'fill';
    }
    get appearanceLegacy() {
        return this.materialStyle === 'legacy';
    }
    get focusedStyle() {
        return this.focused;
    }
    /**
     * @inheritdoc
     */
    ngOnInit() {
        const wrapper = this.hostElement.nativeElement.querySelector('.ngx-markdown-editor-text-editor');
        this.mde = new MarkdownEditor(wrapper, this.mapOptions(this.options));
        this.contentChange.emitObservable(fromCmEvent(this.mde.cm, 'changes'));
        this.cursorActivity.emitObservable(fromCmEvent(this.mde.cm, 'cursorActivity'));
        this.editorFocus.emitObservable(fromCmEvent(this.mde.cm, 'focus'));
        this.editorBlur.emitObservable(fromCmEvent(this.mde.cm, 'blur'));
        this.toolbarService.defineDefaultItems(this);
        this.statusbarService.defineDefaultItems(this.mde);
        // Necessary to apply `this.mde` instance to default toolbar items
        // as `ngOnChanges()` is executed before `ngOnInit()`.
        this.ngOnChanges({ data: new SimpleChange(undefined, this.data, true) });
        this.mde.cm.clearHistory();
        fromCmEvent(this.mde.cm, 'focus').subscribe(() => {
            this.focused = true;
        });
        fromCmEvent(this.mde.cm, 'blur').subscribe(() => {
            if (!this.blockBlur)
                this.focused = false;
        });
        fromCmEvent(this.mde.cm, 'cursorActivity').subscribe(() => {
            this.determineActiveButtons();
        });
    }
    /**
     * @inheritdoc
     */
    ngOnChanges(changes) {
        if (this.mde) {
            if (this.showToolbar) {
                this.applyToolbarItems();
            }
            if (this.showStatusbar) {
                this.applyStatusbarItems();
            }
            this.applyDisabled();
            this.mde.setOptions(this.mapOptions(this.options));
            if (changes.data) {
                this.mde.setContent(changes.data.currentValue);
            }
            this.createTooltips();
            this.determineActiveButtons();
            this.setCodeMirrorClasses();
            this.applyMaterialStyle();
        }
    }
    /**
     * @inheritdoc
     */
    ngOnDestroy() {
        this.shortcutResetter.next();
        this.shortcutResetter.complete();
    }
    /**
     * Toggles the full-size preview.
     */
    togglePreview() {
        this.showPreview = !this.showPreview;
        this.showSideBySidePreview = false;
        if (this.showPreview) {
            // Necessary to wait until Angular change detector has finished
            setTimeout(() => { var _a; return (_a = this.hostElement.nativeElement.querySelector('.ngx-markdown-editor-wrapper')) === null || _a === void 0 ? void 0 : _a.focus(); }, 100);
        }
        else {
            // Necessary to wait until Angular change detector has finished
            setTimeout(() => this.mde.focus(), 100);
        }
    }
    /**
     * Toggles the side-by-side preview.
     */
    toggleSideBySidePreview() {
        this.showSideBySidePreview = !this.showSideBySidePreview;
        this.showPreview = false;
        // Timeout necessary until Angular change detector has finished
        setTimeout(() => this.mde.focus(), 100);
    }
    /**
     * Triggered when a toolbar button is clicked.
     *
     * _Not intended to be used outside the component. Only made public for access inside template._
     */
    onButtonClick(item) {
        item.action();
        this.mde.focus();
        this.determineActiveButtons();
    }
    /**
     * Resolves the shortcut for the specified item and appends it to the item's tooltip text,
     * if `shortcutsInTooltips` is enabled.
     *
     * _Not intended to be used outside the component. Only made public for access inside template._
     */
    createTooltip(item) {
        let shortcut = this.mde.getShortcuts()[item.name] || item.shortcut;
        if (item.name === 'undo')
            shortcut = 'Ctrl-Z';
        else if (item.name === 'redo')
            shortcut = 'Shift-Ctrl-Z';
        if (/Mac/.test(navigator.platform))
            shortcut = shortcut === null || shortcut === void 0 ? void 0 : shortcut.replace(/Ctrl/gi, 'Cmd');
        const shortcutString = this.shortcutsInTooltips && shortcut ? ' (' + shortcut + ')' : '';
        return item.tooltip + shortcutString;
    }
    /**
     * Replaces the checkbox dummies rendered inside the preview with actual checkboxes (also see constructor).
     *
     * _Not intended to be used outside the component. Only made public for access inside template._
     */
    replaceCheckboxDummies() {
        var _a;
        (_a = this.markdown) === null || _a === void 0 ? void 0 : _a.element.nativeElement.querySelectorAll('li').forEach((el) => el.childNodes.forEach((node) => {
            var _a, _b;
            if (node.nodeType === 3) {
                if (/^\[ \] /.test(node.nodeValue || '')) {
                    const input = document.createElement('input');
                    input.setAttribute('type', 'checkbox');
                    input.setAttribute('disabled', '');
                    el.insertBefore(input, node);
                    node.nodeValue = ((_a = node.nodeValue) === null || _a === void 0 ? void 0 : _a.replace(/^\[ \]/, '')) || null;
                }
                else if (/^\[x\] /.test(node.nodeValue || '')) {
                    const input = document.createElement('input');
                    input.setAttribute('type', 'checkbox');
                    input.setAttribute('disabled', '');
                    input.setAttribute('checked', '');
                    el.insertBefore(input, node);
                    node.nodeValue = ((_b = node.nodeValue) === null || _b === void 0 ? void 0 : _b.replace(/^\[x\]/, '')) || null;
                }
            }
        }));
    }
    /**
     * Maps `NgxMdeOptions` to `MarkdownEditorOptions`.
     */
    mapOptions(options) {
        if (!options) {
            return undefined;
        }
        const getMarkdownGuideUrl = (url) => {
            if (!url)
                return undefined;
            if (typeof url === 'string') {
                return url;
            }
            else {
                return url[this.language] || url.default;
            }
        };
        const markupTheme = options.markupThemes || [];
        let editorThemes = options.editorThemes || [];
        if (this.materialStyle) {
            editorThemes = editorThemes ? editorThemes.concat('mde-material') : ['mde-material'];
        }
        else {
            if (editorThemes) {
                const index = editorThemes.findIndex((t) => t === 'mde-material');
                if (index > -1)
                    editorThemes.splice(index, 1);
            }
        }
        const shortcuts = {};
        for (const actionName in DEFAULT_OPTIONS.shortcuts) {
            if (options.shortcuts) {
                shortcuts[actionName] = options.shortcuts[actionName];
            }
        }
        return Object.assign(Object.assign({}, options), { shortcuts, disabled: this.disabled, themes: editorThemes.concat(markupTheme), markdownGuideUrl: getMarkdownGuideUrl(options.markdownGuideUrl) });
    }
    /**
     * Applies the custom toolbar or the default toolbar as fallback.
     */
    applyToolbarItems() {
        let items;
        if (this.toolbar.length) {
            items = this.toolbar;
        }
        else {
            items = this.toolbarService.DEFAULT_TOOLBAR;
        }
        this.normalizedToolbarItems = [];
        for (const toolbarItem of items) {
            const item = this.getNormalizedItem(toolbarItem);
            if (!item) {
                console.warn(`No default item defined for name "${toolbarItem}"`);
                continue;
            }
            this.addSvgIcon(item);
            this.normalizedToolbarItems.push(item);
        }
        this.applyShortcuts(this.normalizedToolbarItems);
    }
    /**
     * Returns a complete item for all combinations of how a toolbar item can be specified and
     * resolves the current value of internationalized properties. Only returns `undefined` for
     * items specified by name and no such item can be found.
     *
     * In detail, item normalization means (in addition to i18n resolution):
     * - For built-in items specified by name string, resolves the default item.
     * - For built-in items specified partly, completes the object with default values for the missing properties.
     * - For custom items specified partly, completes the object with empty values for the missing properties.
     * - For custom items specified fully, returns as is.
     * - For unknown items specified by name string, returns `undefined`.
     */
    getNormalizedItem(toolbarItem) {
        const getTooltip = (tooltip) => {
            if (typeof tooltip === 'string') {
                return tooltip;
            }
            else {
                return tooltip[this.language] || tooltip.default;
            }
        };
        const getIcon = (icon) => {
            if ('format' in icon) {
                return icon;
            }
            else {
                return icon[this.language] || icon.default;
            }
        };
        if (typeof toolbarItem === 'string') {
            return this.toolbarService.getDefaultItem(toolbarItem);
        }
        else {
            let defaultItem = this.toolbarService.getDefaultItem(toolbarItem.name);
            if (!defaultItem) {
                defaultItem = {
                    name: '',
                    action: () => { },
                    tooltip: '',
                    icon: { format: 'material', iconName: '' },
                    disableOnPreview: false,
                };
            }
            return {
                name: toolbarItem.name,
                action: toolbarItem.action || defaultItem.action,
                shortcut: toolbarItem.shortcut || defaultItem.shortcut,
                isActive: toolbarItem.isActive || defaultItem.isActive,
                tooltip: (toolbarItem.tooltip && getTooltip(toolbarItem.tooltip)) || defaultItem.tooltip,
                icon: (toolbarItem.icon && getIcon(toolbarItem.icon)) || defaultItem.icon,
                disableOnPreview: toolbarItem.disableOnPreview || (defaultItem === null || defaultItem === void 0 ? void 0 : defaultItem.disableOnPreview),
            };
        }
    }
    /**
     * Creates tooltips for all configured toolbar items and stores them in `this.toolbarItemTooltips`.
     */
    createTooltips() {
        this.toolbarItemTooltips = new Array(this.normalizedToolbarItems.length);
        for (let i = 0; i < this.normalizedToolbarItems.length; i++) {
            const item = this.normalizedToolbarItems[i];
            this.toolbarItemTooltips[i] = this.showTooltips ? this.createTooltip(item) : '';
        }
    }
    /**
     * Applies custom shortcuts.
     *
     * For items, whose actions originate in _Markdown Editor Core_, `options.shortcuts` is
     * modified. For items that are specific to _Ngx Markdown Editor_ keybindings are applied to
     * the `<ngx-markdown-editor>` element.
     */
    applyShortcuts(items) {
        var _a, _b, _c, _d;
        if (this.options.shortcutsEnabled === 'none') {
            return;
        }
        const applySetHeadingLevelShortcut = (shortcut) => {
            const s = shortcut.replace(/(\w)-/gi, '$1.').replace(/Ctrl/gi, 'Control').replace(/Cmd/gi, 'Meta');
            return this.hotkeys
                .addKeybinding(this.hostElement.nativeElement, s)
                .pipe(takeUntil(this.shortcutResetter))
                .subscribe(() => {
                this.blockBlur = true;
                this.setHeadingLevelDropdown.open();
                this.setHeadingLevelDropdown.focus();
            });
        };
        const applyShortcut = (shortcut, action) => {
            const s = shortcut.replace(/(\w)-/gi, '$1.').replace(/Ctrl/gi, 'Control').replace(/Cmd/gi, 'Meta');
            return this.hotkeys
                .addKeybinding(this.hostElement.nativeElement, s)
                .pipe(takeUntil(this.shortcutResetter))
                .subscribe(() => {
                action();
                this.determineActiveButtons();
            });
        };
        this.shortcutResetter.next();
        const shortcuts = {};
        const appliedNgxMdeShortcuts = {};
        if (this.options.shortcutsEnabled !== 'customOnly') {
            const previewItem = this.toolbarService.getDefaultItem('togglePreview');
            if (previewItem === null || previewItem === void 0 ? void 0 : previewItem.shortcut) {
                const subscription = applyShortcut(previewItem.shortcut, previewItem.action);
                appliedNgxMdeShortcuts[previewItem.name] = subscription;
            }
            const sideBySidePreviewItem = this.toolbarService.getDefaultItem('toggleSideBySidePreview');
            if (sideBySidePreviewItem === null || sideBySidePreviewItem === void 0 ? void 0 : sideBySidePreviewItem.shortcut) {
                const subscription = applyShortcut(sideBySidePreviewItem.shortcut, sideBySidePreviewItem.action);
                appliedNgxMdeShortcuts[sideBySidePreviewItem.name] = subscription;
            }
        }
        for (const item of items) {
            if (item.name === 'setHeadingLevel' && item.shortcut) {
                const subscription = applySetHeadingLevelShortcut(item.shortcut);
                appliedNgxMdeShortcuts[item.name] = subscription;
            }
            else if (item.name in DEFAULT_OPTIONS.shortcuts) {
                shortcuts[item.name] = item.shortcut;
            }
            else if (item.shortcut) {
                (_a = appliedNgxMdeShortcuts[item.name]) === null || _a === void 0 ? void 0 : _a.unsubscribe();
                const subscription = applyShortcut(item.shortcut, item.action);
                appliedNgxMdeShortcuts[item.name] = subscription;
            }
        }
        for (const actionName in this.options.shortcuts) {
            if (this.options.shortcuts[actionName]) {
                const shortcut = this.options.shortcuts[actionName];
                if (actionName === 'setHeadingLevel') {
                    const item = items.find((i) => i.name === actionName);
                    if (item) {
                        (_b = appliedNgxMdeShortcuts[actionName]) === null || _b === void 0 ? void 0 : _b.unsubscribe();
                        applySetHeadingLevelShortcut(shortcut);
                        item.shortcut = shortcut;
                    }
                }
                else if (actionName in DEFAULT_OPTIONS.shortcuts) {
                    shortcuts[actionName] = shortcut;
                }
                else {
                    const item = items.find((i) => i.name === actionName);
                    const defaultItem = this.toolbarService.getDefaultItem(actionName);
                    if (item) {
                        (_c = appliedNgxMdeShortcuts[actionName]) === null || _c === void 0 ? void 0 : _c.unsubscribe();
                        applyShortcut(shortcut, item.action);
                        item.shortcut = shortcut;
                    }
                    else if (defaultItem) {
                        (_d = appliedNgxMdeShortcuts[actionName]) === null || _d === void 0 ? void 0 : _d.unsubscribe();
                        applyShortcut(shortcut, defaultItem.action);
                    }
                }
            }
        }
        this.options.shortcuts = shortcuts;
    }
    /**
     * Adds the SVG specified inside `item.icon` to the injected `MatIconRegistry` instance.
     */
    addSvgIcon(item) {
        switch (item.icon.format) {
            case 'svgString':
                this.iconRegistry.addSvgIconLiteral(item.icon.iconName, this.domSanitizer.bypassSecurityTrustHtml(item.icon.svgHtmlString));
                break;
            case 'svgFile':
                this.iconRegistry.addSvgIcon(item.icon.iconName, this.domSanitizer.bypassSecurityTrustResourceUrl(item.icon.runtimePath));
                break;
        }
    }
    /**
     * Applies the custom statusbar or the default statusbar as fallback.
     */
    applyStatusbarItems() {
        let items;
        if (this.statusbar.length) {
            items = this.statusbar;
        }
        else {
            items = this.statusbarService.DEFAULT_STATUSBAR;
        }
        this.normalizedStatusbarItems = [];
        for (const toolbarItem of items) {
            const item = this.getNormalizedStatusbarItem(toolbarItem);
            if (!item) {
                console.warn(`No default item defined for name "${toolbarItem}"`);
                continue;
            }
            this.normalizedStatusbarItems.push(item);
        }
    }
    /**
     * Returns a complete item for all combinations of how a status item can be specified and
     * resolves the current value of internationalized properties. Only returns `undefined` for
     * items specified by name and no such item can be found.
     *
     * In detail, item normalization means (in addition to i18n resolution):
     * - For built-in items specified by name string, resolves the default item.
     * - For custom items, returns as is.
     * - For unknown items specified by name string, returns `undefined`.
     */
    getNormalizedStatusbarItem(statusbarItem) {
        const getValue = (value) => {
            if (value instanceof Observable) {
                return value;
            }
            else {
                return value[this.language] || value.default;
            }
        };
        if (typeof statusbarItem === 'string') {
            return this.statusbarService.getDefaultItem(statusbarItem);
        }
        else {
            return {
                name: statusbarItem.name,
                value: getValue(statusbarItem.value),
            };
        }
    }
    /**
     * Executes the `item.isActive()` function for all toolbar items and saves the state in `activeItems`.
     */
    determineActiveButtons() {
        this.activeToolbarItems = new Array(this.normalizedToolbarItems.length);
        for (let i = 0; i < this.normalizedToolbarItems.length; i++) {
            const item = this.normalizedToolbarItems[i];
            if (item.isActive) {
                this.activeToolbarItems[i] = item.isActive();
            }
            else {
                this.activeToolbarItems[i] = false;
            }
        }
    }
    /**
     * Applies the disabled state. If disabled, the full-size preview activated.
     */
    applyDisabled() {
        if (this.disabled) {
            this.showPreview = true;
            this.showSideBySidePreview = false;
            this.shortcutResetter.next();
        }
        else {
            this.showPreview = false;
            this.showSideBySidePreview = false;
        }
    }
    /**
     * Applies the material theme to both the editor and the preview. Particularly adds / removes
     * an underline element known from Angular Material `mat-input` form fields.
     */
    applyMaterialStyle() {
        const codemirror = this.hostElement.nativeElement.querySelector('.CodeMirror');
        if (codemirror) {
            const underline = codemirror.querySelector('.underline');
            if (this.materialStyle) {
                if (!underline) {
                    const newUnderline = document.createElement('div');
                    newUnderline.setAttribute('class', 'underline');
                    const newRipple = document.createElement('div');
                    newRipple.setAttribute('class', 'ripple');
                    newUnderline.append(newRipple);
                    codemirror.append(newUnderline);
                }
            }
            else {
                underline === null || underline === void 0 ? void 0 : underline.remove();
            }
        }
        const preview = this.hostElement.nativeElement.querySelector('.ngx-markdown-editor-preview');
        if (preview) {
            const underline = preview.querySelector('.underline');
            if (this.materialStyle) {
                if (!underline) {
                    const newUnderline = document.createElement('div');
                    newUnderline.setAttribute('class', 'underline');
                    preview.append(newUnderline);
                }
            }
            else {
                underline === null || underline === void 0 ? void 0 : underline.remove();
            }
        }
    }
    /**
     * Adds and removes classes to/from the CodeMirror element according to configuration.
     */
    setCodeMirrorClasses() {
        const codemirror = this.hostElement.nativeElement.querySelector('.CodeMirror');
        if (this.options.lineNumbers) {
            codemirror === null || codemirror === void 0 ? void 0 : codemirror.classList.add('CodeMirror-lineNumbers');
        }
        else {
            codemirror === null || codemirror === void 0 ? void 0 : codemirror.classList.remove('CodeMirror-lineNumbers');
        }
    }
}
MarkdownEditorComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-markdown-editor',
                template: "<div *ngIf=\"label\" class=\"ngx-markdown-editor-label\">\n  {{ label }} <span class=\"required-marker\">{{ required ? '*' : '' }}</span>\n</div>\n<div *ngIf=\"showToolbar\" class=\"ngx-markdown-editor-toolbar\">\n  <ng-container *ngFor=\"let item of normalizedToolbarItems; index as i\" [ngSwitch]=\"item.name\">\n    <ng-template [ngSwitchCase]=\"'|'\">\n      <div class=\"mde-toolbar-separator\"></div>\n    </ng-template>\n\n    <ng-template [ngSwitchCase]=\"'setHeadingLevel'\">\n      <mat-form-field floatLabel=\"never\" [matTooltip]=\"toolbarItemTooltips[i]\">\n        <mat-label>\n          <mat-icon *ngIf=\"item.icon.format === 'material'\">\n            {{ item.icon.iconName }}\n          </mat-icon>\n          <mat-icon *ngIf=\"item.icon.format !== 'material'\" [svgIcon]=\"item.icon.iconName\"> </mat-icon>\n        </mat-label>\n        <div (mousedown)=\"blockBlur = true\">\n          <mat-select\n            [value]=\"activeToolbarItems[i]\"\n            [disabled]=\"disabled || (showPreview && item.disableOnPreview)\"\n            (selectionChange)=\"item.action($event.value)\"\n            (closed)=\"mde.cm.focus(); blockBlur = false\"\n            #setHeadingLevel\n          >\n            <mat-option value=\"0\">---</mat-option>\n            <mat-option *ngFor=\"let j of [1, 2, 3, 4, 5, 6]\" [value]=\"j\">H{{ j }}</mat-option>\n          </mat-select>\n        </div>\n      </mat-form-field>\n    </ng-template>\n\n    <ng-template ngSwitchDefault>\n      <button\n        mat-button\n        (mousedown)=\"blockBlur = true\"\n        (mouseup)=\"blockBlur = false\"\n        (click)=\"onButtonClick(item)\"\n        [matTooltip]=\"toolbarItemTooltips[i]\"\n        [class.active]=\"activeToolbarItems[i]\"\n        [disabled]=\"disabled || (showPreview && item.disableOnPreview)\"\n      >\n        <mat-icon *ngIf=\"item.icon.format === 'material'\">\n          {{ item.icon.iconName }}\n        </mat-icon>\n        <mat-icon *ngIf=\"item.icon.format !== 'material'\" [svgIcon]=\"item.icon.iconName\"> </mat-icon>\n      </button>\n    </ng-template>\n  </ng-container>\n</div>\n<div class=\"ngx-markdown-editor-wrapper\" tabindex=\"-1\">\n  <div [ngStyle]=\"{ display: showPreview ? 'none' : '' }\" class=\"ngx-markdown-editor-text-editor\"></div>\n  <div [ngStyle]=\"{ display: showPreview || showSideBySidePreview ? '' : 'none' }\" class=\"ngx-markdown-editor-preview\">\n    <div class=\"preview-wrapper\">\n      <markdown [data]=\"mde.getContent()\" (ready)=\"replaceCheckboxDummies()\" #markdown></markdown>\n    </div>\n  </div>\n</div>\n<div *ngIf=\"showStatusbar\" class=\"ngx-markdown-editor-statusbar\">\n  <ng-container *ngFor=\"let item of normalizedStatusbarItems\" [ngSwitch]=\"item.name\">\n    <div *ngSwitchCase=\"'|'\" class=\"mde-statusbar-separator\"></div>\n    <div *ngSwitchDefault class=\"mde-statusbar-item\">{{ item.value | async }}</div>\n  </ng-container>\n</div>\n",
                providers: [
                    { provide: MAT_TOOLTIP_DEFAULT_OPTIONS, useValue: ɵ0 },
                    ToolbarService,
                    StatusbarService,
                ],
                encapsulation: ViewEncapsulation.None,
                styles: [".CodeMirror{color:#000;direction:ltr;font-family:monospace;height:300px}.CodeMirror-lines{padding:4px 0}.CodeMirror pre.CodeMirror-line,.CodeMirror pre.CodeMirror-line-like{padding:0 4px}.CodeMirror-gutter-filler,.CodeMirror-scrollbar-filler{background-color:transparent}.CodeMirror-gutters{background-color:#f7f7f7;border-right:1px solid #ddd;white-space:nowrap}.CodeMirror-linenumber{color:#999;min-width:20px;padding:0 3px 0 5px;text-align:right;white-space:nowrap}.CodeMirror-guttermarker{color:#000}.CodeMirror-guttermarker-subtle{color:#999}.CodeMirror-cursor{border-left:1px solid #000;border-right:none;width:0}.CodeMirror div.CodeMirror-secondarycursor{border-left:1px solid silver}.cm-fat-cursor .CodeMirror-cursor{background:#7e7;border:0!important;width:auto}.cm-fat-cursor div.CodeMirror-cursors{z-index:1}.cm-fat-cursor-mark{background-color:rgba(20,255,20,.5)}.cm-animate-fat-cursor,.cm-fat-cursor-mark{-webkit-animation:blink 1.06s steps(1) infinite;animation:blink 1.06s steps(1) infinite}.cm-animate-fat-cursor{background-color:#7e7;border:0;width:auto}@-webkit-keyframes blink{50%{background-color:transparent}}@keyframes blink{50%{background-color:transparent}}.cm-tab{display:inline-block;text-decoration:inherit}.CodeMirror-rulers{bottom:0;left:0;overflow:hidden;position:absolute;right:0;top:-50px}.CodeMirror-ruler{border-left:1px solid #ccc;bottom:0;position:absolute;top:0}.cm-s-default .cm-header{color:#00f}.cm-s-default .cm-quote{color:#090}.cm-negative{color:#d44}.cm-positive{color:#292}.cm-header,.cm-strong{font-weight:700}.cm-em{font-style:italic}.cm-link{text-decoration:underline}.cm-strikethrough{text-decoration:line-through}.cm-s-default .cm-keyword{color:#708}.cm-s-default .cm-atom{color:#219}.cm-s-default .cm-number{color:#164}.cm-s-default .cm-def{color:#00f}.cm-s-default .cm-variable-2{color:#05a}.cm-s-default .cm-type,.cm-s-default .cm-variable-3{color:#085}.cm-s-default .cm-comment{color:#a50}.cm-s-default .cm-string{color:#a11}.cm-s-default .cm-string-2{color:#f50}.cm-s-default .cm-meta,.cm-s-default .cm-qualifier{color:#555}.cm-s-default .cm-builtin{color:#30a}.cm-s-default .cm-bracket{color:#997}.cm-s-default .cm-tag{color:#170}.cm-s-default .cm-attribute{color:#00c}.cm-s-default .cm-hr{color:#999}.cm-s-default .cm-link{color:#00c}.cm-invalidchar,.cm-s-default .cm-error{color:red}.CodeMirror-composing{border-bottom:2px solid}div.CodeMirror span.CodeMirror-matchingbracket{color:#0b0}div.CodeMirror span.CodeMirror-nonmatchingbracket{color:#a22}.CodeMirror-matchingtag{background:rgba(255,150,0,.3)}.CodeMirror-activeline-background{background:#e8f2ff}.CodeMirror{background:#fff;overflow:hidden;position:relative}.CodeMirror-scroll{height:100%;margin-bottom:-50px;margin-right:-50px;outline:none;overflow:scroll!important;padding-bottom:50px;position:relative}.CodeMirror-sizer{border-right:50px solid transparent;position:relative}.CodeMirror-gutter-filler,.CodeMirror-hscrollbar,.CodeMirror-scrollbar-filler,.CodeMirror-vscrollbar{display:none;outline:none;position:absolute;z-index:6}.CodeMirror-vscrollbar{overflow-x:hidden;overflow-y:scroll;right:0;top:0}.CodeMirror-hscrollbar{bottom:0;left:0;overflow-x:scroll;overflow-y:hidden}.CodeMirror-scrollbar-filler{bottom:0;right:0}.CodeMirror-gutter-filler{bottom:0;left:0}.CodeMirror-gutters{left:0;min-height:100%;position:absolute;top:0;z-index:3}.CodeMirror-gutter{display:inline-block;height:100%;margin-bottom:-50px;vertical-align:top;white-space:normal}.CodeMirror-gutter-wrapper{background:none!important;border:none!important;position:absolute;z-index:4}.CodeMirror-gutter-background{bottom:0;position:absolute;top:0;z-index:4}.CodeMirror-gutter-elt{cursor:default;position:absolute;z-index:4}.CodeMirror-gutter-wrapper ::selection{background-color:transparent}.CodeMirror-gutter-wrapper ::-moz-selection{background-color:transparent}.CodeMirror-lines{cursor:text;min-height:1px}.CodeMirror pre.CodeMirror-line,.CodeMirror pre.CodeMirror-line-like{-webkit-tap-highlight-color:transparent;background:transparent;border-radius:0;border-width:0;color:inherit;font-family:inherit;font-size:inherit;font-variant-ligatures:contextual;line-height:inherit;margin:0;overflow:visible;position:relative;white-space:pre;word-wrap:normal;z-index:2}.CodeMirror-wrap pre.CodeMirror-line,.CodeMirror-wrap pre.CodeMirror-line-like{white-space:pre-wrap;word-break:normal;word-wrap:break-word}.CodeMirror-linebackground{bottom:0;left:0;position:absolute;right:0;top:0;z-index:0}.CodeMirror-linewidget{padding:.1px;position:relative;z-index:2}.CodeMirror-rtl pre{direction:rtl}.CodeMirror-code{outline:none}.CodeMirror-gutter,.CodeMirror-gutters,.CodeMirror-linenumber,.CodeMirror-scroll,.CodeMirror-sizer{box-sizing:content-box}.CodeMirror-measure{height:0;overflow:hidden;position:absolute;visibility:hidden;width:100%}.CodeMirror-cursor{pointer-events:none;position:absolute}.CodeMirror-measure pre{position:static}div.CodeMirror-cursors{position:relative;visibility:hidden;z-index:3}.CodeMirror-focused div.CodeMirror-cursors,div.CodeMirror-dragcursors{visibility:visible}.CodeMirror-selected{background:#d9d9d9}.CodeMirror-focused .CodeMirror-selected{background:#d7d4f0}.CodeMirror-crosshair{cursor:crosshair}.CodeMirror-line::selection,.CodeMirror-line>span::selection,.CodeMirror-line>span>span::selection{background:#d7d4f0}.CodeMirror-line::-moz-selection,.CodeMirror-line>span::-moz-selection,.CodeMirror-line>span>span::-moz-selection{background:#d7d4f0}.cm-searching{background-color:#ffa;background-color:rgba(255,255,0,.4)}.cm-force-border{padding-right:.1px}@media print{.CodeMirror div.CodeMirror-cursors{visibility:hidden}}.cm-tab-wrap-hack:after{content:\"\"}span.CodeMirror-selectedtext{background:none}ngx-markdown-editor{box-sizing:border-box;display:flex;flex-direction:column}.ngx-markdown-editor-toolbar .mat-button{border:1px solid transparent;line-height:28px;margin:3px 2px;min-width:30px;padding:0}.ngx-markdown-editor-toolbar .mat-button.active{background-color:rgba(0,0,0,.05);border-color:#a9a9a9}.ngx-markdown-editor-toolbar .mat-form-field{font-size:12px;margin:-10px 3px -15px;min-width:3em;text-align:center;width:3em}.ngx-markdown-editor-toolbar .mat-form-field:first-child{margin-left:10px}.ngx-markdown-editor-toolbar .mat-form-field.mat-form-field-disabled .mat-icon{color:rgba(0,0,0,.26)}.ngx-markdown-editor-toolbar .mde-toolbar-separator{border-left:1px solid #a9a9a9;display:inline-block;height:24px;vertical-align:middle}.ngx-markdown-editor-toolbar .mat-button .mat-icon{font-size:20px;height:20px;width:20px}.ngx-markdown-editor-toolbar .mat-button .mat-icon[data-mat-icon-name=column],.ngx-markdown-editor-toolbar .mat-button .mat-icon[data-mat-icon-name=file_code],.ngx-markdown-editor-toolbar .mat-button .mat-icon[data-mat-icon-name=upload]{height:16px}.ngx-markdown-editor-toolbar .mat-button .mat-icon[data-mat-icon-name=format_heading]{height:13px;margin-top:-2px}.ngx-markdown-editor-toolbar .mat-button .mat-icon[data-mat-icon-name=format_heading_decrease],.ngx-markdown-editor-toolbar .mat-button .mat-icon[data-mat-icon-name=format_heading_increase]{height:18px;margin-top:-2px}.ngx-markdown-editor-toolbar .mat-form-field .mat-icon[data-mat-icon-name=format_heading]{height:13px}.ngx-markdown-editor-toolbar svg{vertical-align:top}.ngx-markdown-editor-wrapper{box-sizing:border-box;display:flex;flex:1 1 100%;flex-direction:row;overflow:hidden}.ngx-markdown-editor-wrapper:focus{outline:none}.ngx-markdown-editor-text-editor{box-sizing:border-box;display:flex;flex:1 1 100%;flex-direction:column}.ngx-markdown-editor-text-editor .CodeMirror{height:100%;word-wrap:break-word}.ngx-markdown-editor-text-editor .CodeMirror .CodeMirror-placeholder{color:rgba(0,0,0,.6)}.ngx-markdown-editor-preview{background-color:#f0f0f0;box-sizing:border-box;flex:1 1 100%;overflow-y:auto;word-wrap:break-word}.ngx-markdown-editor-preview>.preview-wrapper{height:100%;overflow:auto;padding:0 10px}.ngx-markdown-editor-statusbar{color:rgba(0,0,0,.6);font-size:.75em;margin-top:5px;text-align:right}.ngx-markdown-editor-statusbar *{display:inline-block;margin:0 7px}.ngx-markdown-editor-statusbar .mde-statusbar-separator{border-left:1px solid #a9a9a9;height:14px;margin:0 4px;vertical-align:middle}ngx-markdown-editor.default .ngx-markdown-editor-label{font-weight:700;padding-left:8px;text-decoration:underline}ngx-markdown-editor.default .ngx-markdown-editor-wrapper>*{border:1px solid #ddd}ngx-markdown-editor.default .ngx-markdown-editor-wrapper :first-child[style=\"display: none;\"]+*,ngx-markdown-editor.default .ngx-markdown-editor-wrapper>:first-child{border-bottom-left-radius:5px;border-top-left-radius:5px}ngx-markdown-editor.default .ngx-markdown-editor-wrapper>:last-child{border-bottom-right-radius:5px;border-top-right-radius:5px}ngx-markdown-editor.default .ngx-markdown-editor-wrapper .CodeMirror{border-radius:5px}ngx-markdown-editor.default .ngx-markdown-editor-wrapper .CodeMirror .CodeMirror-scroll{padding-left:8px;padding-top:8px}ngx-markdown-editor.default .ngx-markdown-editor-wrapper .CodeMirror.CodeMirror-lineNumbers .CodeMirror-scroll{padding-left:0}.CodeMirror.cm-s-preview-like-markup{font-family:Roboto,Helvetica Neue,sans-serif}.CodeMirror.cm-s-preview-like-markup .cm-header{color:#000;font-weight:700}.CodeMirror.cm-s-preview-like-markup .cm-header.cm-header-1{font-size:2em}.CodeMirror.cm-s-preview-like-markup .cm-header.cm-header-2{font-size:1.5em}.CodeMirror.cm-s-preview-like-markup .cm-header.cm-header-3{font-size:1.17em}.CodeMirror.cm-s-preview-like-markup .cm-header.cm-header-4{font-size:1em}.CodeMirror.cm-s-preview-like-markup .cm-header.cm-header-5{font-size:.83em}.CodeMirror.cm-s-preview-like-markup .cm-header.cm-header-6{font-size:.67em}.CodeMirror.cm-s-preview-like-markup .cm-code{font-family:monospace}.CodeMirror.cm-s-preview-like-markup .cm-link-text{color:#0366d6;text-decoration:underline}.CodeMirror.cm-s-preview-like-markup .cm-link-href{color:#afafaf;font-size:.8em;text-decoration:none}"]
            },] }
];
MarkdownEditorComponent.ctorParameters = () => [
    { type: MatIconRegistry },
    { type: DomSanitizer },
    { type: Keybindings },
    { type: ElementRef },
    { type: MarkdownService },
    { type: ToolbarService, decorators: [{ type: Host }] },
    { type: StatusbarService, decorators: [{ type: Host }] }
];
MarkdownEditorComponent.propDecorators = {
    data: [{ type: Input }],
    options: [{ type: Input }],
    toolbar: [{ type: Input }],
    statusbar: [{ type: Input }],
    language: [{ type: Input }],
    label: [{ type: Input }],
    required: [{ type: Input }],
    materialStyle: [{ type: Input }],
    disabled: [{ type: Input }],
    showToolbar: [{ type: Input }],
    showStatusbar: [{ type: Input }],
    showTooltips: [{ type: Input }],
    shortcutsInTooltips: [{ type: Input }],
    contentChange: [{ type: Output }],
    cursorActivity: [{ type: Output }],
    editorFocus: [{ type: Output }],
    editorBlur: [{ type: Output }],
    setHeadingLevelDropdown: [{ type: ViewChild, args: ['setHeadingLevel',] }],
    markdown: [{ type: ViewChild, args: ['markdown',] }],
    disabledStyle: [{ type: HostBinding, args: ['class.disabled',] }],
    default: [{ type: HostBinding, args: ['class.default',] }],
    material: [{ type: HostBinding, args: ['class.material',] }],
    class: [{ type: HostBinding, args: ['class',] }],
    appearanceStandard: [{ type: HostBinding, args: ['class.appearance-standard',] }],
    appearanceFill: [{ type: HostBinding, args: ['class.appearance-fill',] }],
    appearanceLegacy: [{ type: HostBinding, args: ['class.appearance-legacy',] }],
    focusedStyle: [{ type: HostBinding, args: ['class.focused',] }]
};

class MarkdownEditorModule {
    static forRoot(config) {
        var _a, _b, _c;
        return {
            ngModule: MarkdownEditorModule,
            providers: [
                ((_a = config === null || config === void 0 ? void 0 : config.previewConfig) === null || _a === void 0 ? void 0 : _a.markedOptions) || [],
                {
                    provide: SECURITY_CONTEXT,
                    useValue: (_c = (_b = config === null || config === void 0 ? void 0 : config.previewConfig) === null || _b === void 0 ? void 0 : _b.sanitize) !== null && _c !== void 0 ? _c : SecurityContext.HTML,
                },
            ],
        };
    }
    static forChild() {
        return {
            ngModule: MarkdownEditorModule,
        };
    }
}
MarkdownEditorModule.decorators = [
    { type: NgModule, args: [{
                declarations: [MarkdownEditorComponent],
                imports: [
                    CommonModule,
                    BrowserModule,
                    BrowserAnimationsModule,
                    HttpClientModule,
                    MatIconModule,
                    MatButtonModule,
                    MatFormFieldModule,
                    MatSelectModule,
                    MatTooltipModule,
                    MarkdownModule.forRoot(),
                ],
                exports: [MarkdownEditorComponent],
            },] }
];

/*
 * Public API Surface of ngx-markdown-editor
 */

/**
 * Generated bundle index. Do not edit.
 */

export { MarkdownEditorComponent, MarkdownEditorModule, fromCmEvent, ToolbarService as ɵa, StatusbarService as ɵb, Keybindings as ɵc };
//# sourceMappingURL=mdefy-ngx-markdown-editor.js.map
